@@ -646,6 +646,7 @@ class ColorCycleDialog(QDialog):
         self.color_list.setDragDropMode(QTreeWidget.DragDropMode.NoDragDrop)  # Disable drag-and-drop
         self.color_list.itemSelectionChanged.connect(self._on_selection_changed)
         self.color_list.itemChanged.connect(self._on_item_changed)
+        self.color_list.itemDoubleClicked.connect(self._edit_color)
         left_layout.addWidget(self.color_list)
         
         # Color control buttons
@@ -1003,14 +1004,4 @@ class ColorCycleDialog(QDialog):
 
 
 if __name__ == "__main__":
-    # TEST
-    # test the colordialog
-    import sys
-    from PyQt6.QtWidgets import QApplication
-    app = QApplication(sys.argv)
-    dialog = ColorCycleDialog()
-
-    # generate test data
-    y = np.random.rand(100)
-    dialog.set_preview_data(y)
-    dialog.exec()
\ No newline at end of file
+    pass
\ No newline at end of file
