@@ -8,12 +8,13 @@ MAX IV Laboratory, Lund University, Sweden
 This module provides the main application window for plotting azimuthally integrated data,
 including loading files, displaying heatmaps and patterns, and managing auxiliary data.
 """
+from operator import index
 import sys
 import os
 import numpy as np
 from PyQt6.QtWidgets import (QApplication, QMainWindow, QVBoxLayout, QHBoxLayout, QWidget, QDockWidget,
-                             QSizePolicy, QFileDialog, QMessageBox, QProgressDialog, QCheckBox)
-from PyQt6.QtGui import QAction, QIcon
+                             QSizePolicy, QFileDialog, QMessageBox, QProgressDialog, QCheckBox, QSplashScreen)
+from PyQt6.QtGui import QAction, QIcon, QPixmap
 from PyQt6 import QtCore
 import pyqtgraph as pg
 import h5py as h5
@@ -35,7 +36,7 @@ if parent_dir not in sys.path:
 from plaid.trees import FileTreeWidget, CIFTreeWidget
 from plaid.dialogs import H5Dialog, ExportSettingsDialog, ColorCycleDialog
 from plaid.reference import Reference
-from plaid.plot_widgets import HeatmapWidget, PatternWidget, AuxiliaryPlotWidget
+from plaid.plot_widgets import HeatmapWidget, PatternWidget, AuxiliaryPlotWidget, CorrelationMapWidget
 from plaid.misc import q_to_tth, tth_to_q
 from plaid.data_containers import AzintData, AuxData
 from plaid import __version__ as CURRENT_VERSION
@@ -69,6 +70,15 @@ import plaid.resources
 # - save additional settings like default path(s), dock widget positions, etc.
 # - optimize memory usage and performance for large datasets
 # - add more tooltips
+# - add a "reduction factor" option to reduce the effective time resolution of the data (I, I0, and aux data)
+# - add a "show auto-correlation map" option as a separate dock widget
+#    > Perhaps with an option to click a pixel in the correlation map and see the corresponding pattern(s)?
+# - add a "show map" option
+#    > show a map in a separate dock widget
+#    > change the line positions and pattern when a pixel is clicked
+#    > select roi in the pattern widget
+#    > get the pixel coordinates the an nxsample/nxtransformation group
+#      or ask the user to specify a rectangle shape
 
 ALLOW_EXPORT_ALL_PATTERNS = True
 PLOT_I0 = True
@@ -192,6 +202,27 @@ def clear_all_settings():
     settings.clear()
     print("All settings cleared.")
 
+def _get_desktop_path():
+    """Get the path to the user's desktop in a cross-platform way."""
+    if os.name != 'nt':
+        return os.path.join(os.path.expanduser("~"), "Desktop")
+    else:
+        import ctypes
+        from ctypes import wintypes
+        buf = ctypes.create_unicode_buffer(wintypes.MAX_PATH)
+        ctypes.windll.shell32.SHGetFolderPathW(None, 0x0000, None, 0, buf)  # 0x0000 = CSIDL_DESKTOP
+        return os.path.abspath(buf.value)
+
+def _get_default_path():
+    """
+    Get a sensible default path for file dialogs.
+    returns the current working directory, unless it is the same as the script path,
+    in which case it returns the user's home directory.
+    """
+    default = os.getcwd()
+    if default == os.path.dirname(__file__):
+        default = os.path.expanduser("~")
+    return os.path.abspath(default)
 
 class MainWindow(QMainWindow):
     """plaid - Main application window for plotting azimuthally integrated data."""
@@ -246,6 +277,7 @@ class MainWindow(QMainWindow):
         self._init_file_tree()
         self._init_cif_tree()
         self._init_auxiliary_plot()
+        self._init_correlation_map()
         # Add the dock widgets to the main window
         self._init_dock_widget_settings()
 
@@ -273,6 +305,7 @@ class MainWindow(QMainWindow):
 
         # Check for updates on startup (non-blocking)
         self._check_for_updates_on_startup()
+        self._check_if_first_run()
 
     def _init_file_tree(self):
         """Initialize the file tree widget. Called by self.__init__()."""
@@ -306,6 +339,19 @@ class MainWindow(QMainWindow):
         auxiliary_plot_dock.setWidget(self.auxiliary_plot)
         self.auxiliary_plot_dock = auxiliary_plot_dock
 
+    def _init_correlation_map(self):
+        """Initialize the correlation map widget. Called by self.__init__()."""
+        self.correlation_map = CorrelationMapWidget(self)
+        # create a dock widget for the correlation map
+        correlation_map_dock = QDockWidget("Auto-correlation Map", self)
+        correlation_map_dock.setAllowedAreas(QtCore.Qt.DockWidgetArea.LeftDockWidgetArea | QtCore.Qt.DockWidgetArea.RightDockWidgetArea)
+        correlation_map_dock.setFeatures(QDockWidget.DockWidgetFeature.DockWidgetMovable | QDockWidget.DockWidgetFeature.DockWidgetFloatable | QDockWidget.DockWidgetFeature.DockWidgetClosable)
+        correlation_map_dock.setWidget(self.correlation_map)
+        self.correlation_map_dock = correlation_map_dock
+        self.correlation_map_dock.setFloating(True)
+        # hide the correlation map dock by default
+        self.correlation_map_dock.hide()
+
     def _init_dock_widget_settings(self):
         """Initialize the dock widgets based on previously saved settings. Called by self.__init__()."""
         # get the current dock widget settings (if any)
@@ -327,6 +373,7 @@ class MainWindow(QMainWindow):
             self.addDockWidget(QtCore.Qt.DockWidgetArea.LeftDockWidgetArea, self.file_tree_dock)
             self.addDockWidget(QtCore.Qt.DockWidgetArea.LeftDockWidgetArea, self.cif_tree_dock)
             self.addDockWidget(QtCore.Qt.DockWidgetArea.LeftDockWidgetArea, self.auxiliary_plot_dock)
+        self.addDockWidget(QtCore.Qt.DockWidgetArea.RightDockWidgetArea, self.correlation_map_dock)
         self.setDockOptions(QMainWindow.DockOption.AnimatedDocks)
 
     def _init_connections(self):
@@ -354,6 +401,9 @@ class MainWindow(QMainWindow):
         self.auxiliary_plot.sigVLineMoved.connect(self.vline_moved)
         self.auxiliary_plot.sigAuxHovered.connect(self.update_status_bar_aux)
 
+        self.correlation_map_dock.visibilityChanged.connect(self.update_correlation_map)
+        self.correlation_map.sigImageDoubleClicked.connect(self.correlation_map_double_clicked)
+
     def _init_menu_bar(self):
         """Initialize the menu bar with the necessary menus and actions. Called by self.__init__()."""
         # Create a menu bar
@@ -430,6 +480,11 @@ class MainWindow(QMainWindow):
         toggle_auxiliary_plot_action = self.auxiliary_plot_dock.toggleViewAction()
         toggle_auxiliary_plot_action.setText("Show &Auxiliary Plot")
         view_menu.addAction(toggle_auxiliary_plot_action)
+        # Add an action to toggle the correlation map visibility
+        toggle_correlation_map_action = self.correlation_map_dock.toggleViewAction()
+        toggle_correlation_map_action.setText("Show Auto-correlation &Map")
+        view_menu.addAction(toggle_correlation_map_action)
+
         # add a separator
         view_menu.addSeparator()
         # add a toggle Q action
@@ -496,6 +551,13 @@ class MainWindow(QMainWindow):
         update_action.triggered.connect(self.check_for_updates_manual)
         update_action.setEnabled(HAS_UPDATE_CHECKER)  # Only enable if requests is available
         help_menu.addAction(update_action)
+
+        # Add action to create a desktop shortcut (Windows only)
+        if os.name == 'nt':
+            shortcut_action = QAction("Create &Desktop Shortcut", self)
+            shortcut_action.setToolTip("Create a desktop shortcut for plaid")
+            shortcut_action.triggered.connect(self.create_shortcut)
+            help_menu.addAction(shortcut_action)
         
         # Add separator
         help_menu.addSeparator()
@@ -622,7 +684,8 @@ class MainWindow(QMainWindow):
             if self.file_tree.files and self.file_tree.files[-1] is not None:
                 default_dir = os.path.dirname(self.file_tree.files[-1])
             else:
-                default_dir = os.path.expanduser("~")
+                # default_dir = os.path.expanduser("~")
+                default_dir = _get_default_path()
             file_path, ok = QFileDialog.getOpenFileName(self, "Select Azimuthal Integration File", default_dir, "HDF5 Files (*.h5);;All Files (*)")
             if not ok or not file_path:
                 return
@@ -752,6 +815,8 @@ class MainWindow(QMainWindow):
         if not aux_plot_key is None:
             # if a selected item is provided, add the auxiliary plot for that item
             self.add_auxiliary_plot(aux_plot_key)
+        
+        self.update_correlation_map(self.correlation_map_dock.isVisible())
 
     def hline_moved(self, index, pos):
         """Handle the horizontal line movement in the heatmap."""
@@ -783,7 +848,8 @@ class MainWindow(QMainWindow):
         if self.cif_tree.files and self.cif_tree.files[-1] is not None:
             default_dir = os.path.dirname(self.cif_tree.files[-1])
         else:
-            default_dir = os.path.expanduser("~")
+            # default_dir = os.path.expanduser("~")
+            default_dir = _get_default_path()
         file_path, ok = QFileDialog.getOpenFileName(self, "Select Crystallographic Information File", default_dir, "CIF Files (*.cif);;All Files (*)")
         if not ok or not file_path:
             return
@@ -1168,6 +1234,33 @@ class MainWindow(QMainWindow):
         # inform the user that the export is done
         QMessageBox.information(self, "Complete", f"Complete!\nExported {self.azint_data.shape[0]} patterns to:\n{directory}")
 
+    def update_correlation_map(self, is_checked):
+        """Update the correlation map when the correlation map checkbox is toggled."""
+        if is_checked and self.azint_data.I is not None:
+            # check if the correlation map is already calculated for the current data
+            if not self.azint_data.fnames == self.correlation_map.fnames:
+                self.correlation_map.set_data(self.azint_data.get_I())
+                self.correlation_map.fnames = self.azint_data.fnames
+
+    def correlation_map_double_clicked(self, pos):
+        """Handle double click events on the correlation map."""
+        if self.correlation_map.fnames:
+            # Get the index of the clicked position
+            x, y = pos
+
+            # Ensure that at least to horizontal lines exist in the heatmap
+            if len(self.heatmap.h_lines) < 2:
+                for i in range(2 - len(self.heatmap.h_lines)):
+                    self.add_pattern((0,i))
+
+            # move the last two horizontal and vertical lines to the selected positions
+            for i in range(2):
+                index = len(self.heatmap.h_lines) - 2 + i
+                self.update_pattern(index, pos[i])
+                self.heatmap.set_h_line_pos(index, pos[i])
+                self.auxiliary_plot.set_v_line_pos(index, pos[i])
+
+
     def dragEnterEvent(self, event):
         """Handle drag and drop events for the main window."""
         if event.mimeData().hasUrls():
@@ -1198,11 +1291,12 @@ class MainWindow(QMainWindow):
             x = self.heatmap.x
             # y = np.arange(I.shape[0])
             self.heatmap.set_data(x, I.T)
-
+        elif event.key() == QtCore.Qt.Key.Key_C:
+            # Show/hide the correlation map
+            self.correlation_map_dock.setVisible(not self.correlation_map_dock.isVisible())
         elif event.key() == QtCore.Qt.Key.Key_Q:
             # Toggle between q and 2theta
             self.toggle_q()
-
         elif event.key() == QtCore.Qt.Key.Key_Up:
             # Move the selected line one increment up
             self.heatmap.move_active_h_line(1)
@@ -1220,34 +1314,6 @@ class MainWindow(QMainWindow):
 
         # # DEBUG
         elif event.key() == QtCore.Qt.Key.Key_Space:
-            
-            # h5dialog = H5Dialog(self, self.azint_data.fnames[0])
-            # if h5dialog.exec_1d_2d_pair():
-            #     selected = h5dialog.get_selected_items()
-            #     axis = [item for item in selected if not "×" in item[2]][0]
-            #     signal = [item for item in selected if "×" in item[2]][0]
-            # print('axis:', axis)
-            # print('signal:', signal)
-            # fname = self.azint_data.fnames[0] 
-            # dialog = H5Dialog(self, fname)
-            # if not dialog.exec_1d_2d_pair():
-            #     print( None, None, None, None)
-
-            # selected = dialog.get_selected_items() # list of tuples with (alias, full_path, shape)
-            # axis = [item for item in selected if not "×" in item[2]][0] 
-            # signal = [item for item in selected if "×" in item[2]][0]
-            # # Check if the shape of the axis and signal match
-            # if not axis[2] in signal[2].split("×")[1]:
-            #     print(f"Error: The shape of the axis {axis[2]} does not match the shape of the signal {signal[2]}.")
-            #     print( None, None, None, None)
-            # with h5.File(fname, 'r') as f:
-            #     x = f[axis[1]][:]
-            #     I = f[signal[1]][:]
-            #     # attempt to guess if the axis is q or 2theta
-            #     is_q = 'q' in axis[0].lower() or 'q' in f[axis[1]].attrs.get('long_name', '').lower()
-            # print(x, I, is_q, None)
-            
-            # ## TEST COLORDIALOG
             pass
 
     def show_color_cycle_dialog(self):
@@ -1271,6 +1337,7 @@ class MainWindow(QMainWindow):
         settings.beginGroup("DockWidgets")
         # Find all dock widgets and sort them by area
         dock_widgets = self.findChildren(QDockWidget)
+        dock_widgets = [dock for dock in dock_widgets if dock.windowTitle() in ['File Tree', 'CIF Tree', 'Auxiliary Plot']]
         left = [dock for dock in dock_widgets if self.dockWidgetArea(dock) == QtCore.Qt.DockWidgetArea.LeftDockWidgetArea]
         right = [dock for dock in dock_widgets if self.dockWidgetArea(dock) == QtCore.Qt.DockWidgetArea.RightDockWidgetArea]
         # Sort the dock widgets by their y position
@@ -1432,6 +1499,96 @@ class MainWindow(QMainWindow):
         # Use QTimer to make it non-blocking
         QtCore.QTimer.singleShot(100, perform_check)
 
+    def _check_if_first_run(self):
+        """Check if this is the first run of the application."""
+        settings = QtCore.QSettings("plaid", "plaid")
+        first_run = settings.value("first_run", True, type=bool)
+        if first_run:
+            def show_welcome_message():
+                # Show a welcome message and ask if the user wish to create a desktop shortcut
+                welcome_text = (
+                    "<h2>Welcome to plaid!</h2>"
+                    "<p>Thank you for using plaid - plot azimuthally integrated data.</p>"
+                    "<p>You can find help to some of the basic functionalities in the 'Help' menu.</p>"
+                )
+                if os.name == 'nt':  # Only ask for shortcut creation on Windows
+                    welcome_text += "<p>Do you wish to create a desktop shortcut?</p>"
+                    reply = QMessageBox.question(self, "Welcome", welcome_text)
+                    if reply == QMessageBox.StandardButton.Yes:
+                        self.create_shortcut()
+                else:
+                    QMessageBox.information(self, "Welcome", welcome_text)
+            # delay the welcome message by 1 second to avoid blocking startup
+            QtCore.QTimer.singleShot(1000, show_welcome_message)
+            # Set first_run to False for future runs
+            settings.setValue("first_run", False)
+
+    def create_ico_from_resource(self, target_path=None):
+        """
+        Save the application icon from Qt resources as an .ico file.
+        If target_path is None, saves to the directory containing plaid.py and resources.py.
+        Returns the path to the saved .ico file.
+        """
+        if target_path is None:
+            # Save to the same directory as this file (plaid.py)
+            base_dir = os.path.dirname(os.path.abspath(__file__))
+            target_path = os.path.join(base_dir, 'plaid.ico')
+        icon = QIcon(':/icons/plaid.png')
+        pixmap = icon.pixmap(256, 256)
+        pixmap.save(target_path, 'ICO')
+        return target_path
+
+    def create_shortcut(self):
+        """
+        Create a desktop shortcut to launch the application.
+        WINDOWS ONLY
+        """
+        if os.name != 'nt':
+            QMessageBox.warning(self, "Unsupported OS", "Shortcut creation is only supported on Windows.")
+            return
+        # Find the user's desktop path
+        shortcut_path = _get_desktop_path()
+        if not os.path.exists(shortcut_path):
+            # if the desktop path does not exist, prompt the user
+            # for a different location
+            shortcut_path = ""
+            shortcut_path = QFileDialog.getExistingDirectory(self, "Select Directory to Create Shortcut", os.path.expanduser("~"))
+            if not shortcut_path:
+                return
+            shortcut_path = os.path.abspath(shortcut_path)
+        shortcut_path = os.path.join(shortcut_path, 'Plaid.lnk')
+
+        # Find the current Python interpreter
+        python_exe = sys.executable
+
+        # Find the absolute path to plaid.py
+        plaid_py = os.path.abspath(__file__)
+
+        # Find the .ico file (use your create_ico_from_resource method)
+        ico_path = self.create_ico_from_resource()
+
+        # Set working directory to user profile
+        working_dir = os.environ['USERPROFILE']
+
+        # Compose PowerShell command to create the shortcut
+        powershell_cmd = (
+            f"$s=(New-Object -COM WScript.Shell).CreateShortcut('{shortcut_path}');"
+            f"$s.TargetPath='{python_exe}';"
+            f"$s.Arguments='\"{plaid_py}\"';"
+            f"$s.IconLocation='{ico_path}';"
+            f"$s.WorkingDirectory='{working_dir}';"
+            f"$s.WindowStyle=7;"  # 7 = Minimized
+            "$s.Save()"
+        )
+
+        # Run the command
+        os.system(f'powershell -NoProfile -Command "{powershell_cmd}"')
+
+        if os.path.exists(shortcut_path):
+            QMessageBox.information(self, "Shortcut Created", f"Shortcut created:\n{shortcut_path}")
+        else:
+            QMessageBox.critical(self, "Error", "Failed to create shortcut.")
+
     def show(self):
         """Override the show method to update the pattern geometry."""
         super().show()
@@ -1496,6 +1653,12 @@ def main():
 
     # Create the application and main window
     app = QApplication(sys.argv)
+
+    # Create and show the splash screen
+    splash_pix = QPixmap(":/icons/plaid.png")  # Use your resource or a file path
+    splash = QSplashScreen(splash_pix)
+    splash.show()
+
     # app.setStyle("Fusion")
     # get the application palette colors
     foreground_color = app.palette().text().color().darker(150).name()
@@ -1513,6 +1676,8 @@ def main():
             window.open_file(file)
     # show the main window
     window.show()
+    splash.finish(window)
+
     sys.exit(app.exec())
 
 if __name__ == "__main__":
