@@ -12,7 +12,7 @@ from operator import index
 import numpy as np
 from PyQt6.QtWidgets import QVBoxLayout, QHBoxLayout, QWidget, QToolBar
 from PyQt6 import QtCore
-from PyQt6.QtGui import QColor
+from PyQt6.QtGui import QColor, QTransform
 import pyqtgraph as pg
 
 colors = [
@@ -563,6 +563,7 @@ class AuxiliaryPlotWidget(QWidget):
     def __init__(self, parent=None):
         super().__init__(parent)
         self.v_lines = []
+        self.plot_data_items = []
         self.n = None  # Number of data points in the x-axis
         self.color_cycle = colors
         # Create a layout
@@ -589,7 +590,8 @@ class AuxiliaryPlotWidget(QWidget):
         if color is None:
             color = self.color_cycle[len(self.plot_item.items) % len(self.color_cycle)]
         x = np.arange(len(y))
-        self.plot_item.plot(x, y, pen=color, name=label if label else 'Auxiliary Plot')
+        plot_data_item = self.plot_item.plot(x, y, pen=color, name=label if label else 'Auxiliary Plot')
+        self.plot_data_items.append(plot_data_item)
         self.n = len(y)
 
         # make sure the vlines are re-added if the plot has been cleared
@@ -664,6 +666,7 @@ class AuxiliaryPlotWidget(QWidget):
         """Set the color cycle for the plot items."""
         self.color_cycle = color_cycle
         self._update_line_colors()
+        self._update_plot_colors()
 
     def _update_line_colors(self):
         """Update the colors of the line items based on the color cycle."""
@@ -674,9 +677,16 @@ class AuxiliaryPlotWidget(QWidget):
             pen.setColor(color)
             line.setPen(pen)
 
+    def _update_plot_colors(self):
+        """Update the colors of the plot items based on the color cycle."""
+        for i, pdi in enumerate(self.plot_data_items):
+            color = QColor(self.color_cycle[i % len(self.color_cycle)])
+            pdi.setPen(color)
+
     def clear_plot(self):
         """Clear the auxiliary plot."""
         self.plot_item.clear()
+        self.plot_data_items = []
 
     def clear(self):
         """Clear the auxiliary plot and vertical lines."""
@@ -687,6 +697,65 @@ class AuxiliaryPlotWidget(QWidget):
         self.v_lines = []
         #self.addVLine()
 
+class CorrelationMapWidget(QWidget):
+    """
+    A widget to display a correlation map.
+    """
+    sigImageDoubleClicked = QtCore.pyqtSignal(object)  # Signal emitted when the image is double-clicked
+    def __init__(self, parent=None):
+        super().__init__(parent)
+        self.fnames = []  # used to keep track of which dataset is used
+        self.n = None  # Number of data points in the x-axis
+        # Create a layout
+        layout = QHBoxLayout(self)
+
+        # Create a pyqtgraph PlotWidget
+        self.plot_widget = pg.PlotWidget()
+        layout.addWidget(self.plot_widget,1)
+
+        # Create a image item for the heatmap
+        self.image_item = pg.ImageItem()
+        self.plot_widget.addItem(self.image_item)
+        self.plot_widget.setLimits(minXRange=3)
+
+        tr = QTransform()
+        tr.translate(-0.5, -0.5)
+        self.image_item.setTransform(tr)       
+
+        self.x_axis = self.plot_widget.getPlotItem().getAxis('bottom')
+        self.y_axis = self.plot_widget.getPlotItem().getAxis('left')
+
+        self.x_axis.setLabel("frame number #")
+        self.y_axis.setLabel("frame number #")
+
+        # Create a histogram widget
+        self.histogram = pg.HistogramLUTWidget()
+        self.histogram.setImageItem(self.image_item)
+        self.histogram.item.gradient.loadPreset('viridis')
+        layout.addWidget(self.histogram,0)
+
+        self.plot_widget.getPlotItem().mouseDoubleClickEvent = self.image_double_clicked
+
+    def set_data(self, z):
+        """Set the data for the correlation map."""
+        if z is None:
+            return
+        # compute the correlation matrix
+        im = np.corrcoef(z)
+        self.image_item.setImage(im)
+        n = im.shape[0]
+        self.n = n
+        # update the limits of the plot
+        self.plot_widget.setLimits(xMin=-n*.1, xMax=n*1.1, yMin=-n*0.1, yMax=n*1.1)
+
+    def image_double_clicked(self, event):
+        """Handle the double click event on the image item."""
+        if event.button() == QtCore.Qt.MouseButton.LeftButton and self.n is not None:
+            pos = self.plot_widget.getPlotItem().vb.mapSceneToView(event.pos())
+            x = int(np.clip(pos.x()+0.5, 0, self.n - 1))
+            y = int(np.clip(pos.y()+0.5, 0, self.n - 1))
+            self.sigImageDoubleClicked.emit((x, y))
+
 
 if __name__ == "__main__":
     pass
\ No newline at end of file
